@@ -9,11 +9,13 @@
 
 #include "alloc-util.h"
 #include "ask-password-api.h"
+#include "build.h"
 #include "chase-symlinks.h"
 #include "copy.h"
 #include "creds-util.h"
 #include "dissect-image.h"
 #include "env-file.h"
+#include "errno-util.h"
 #include "fd-util.h"
 #include "fileio.h"
 #include "fs-util.h"
@@ -96,7 +98,7 @@ static bool press_any_key(void) {
 }
 
 static void print_welcome(void) {
-        _cleanup_free_ char *pretty_name = NULL, *ansi_color = NULL;
+        _cleanup_free_ char *pretty_name = NULL, *os_name = NULL, *ansi_color = NULL;
         static bool done = false;
         const char *pn, *ac;
         int r;
@@ -110,12 +112,13 @@ static void print_welcome(void) {
         r = parse_os_release(
                         arg_root,
                         "PRETTY_NAME", &pretty_name,
+                        "NAME", &os_name,
                         "ANSI_COLOR", &ansi_color);
         if (r < 0)
                 log_full_errno(r == -ENOENT ? LOG_DEBUG : LOG_WARNING, r,
                                "Failed to read os-release file, ignoring: %m");
 
-        pn = isempty(pretty_name) ? "Linux" : pretty_name;
+        pn = os_release_pretty_name(pretty_name, os_name);
         ac = isempty(ansi_color) ? "0" : ansi_color;
 
         if (colors_enabled())
@@ -161,7 +164,7 @@ static int show_menu(char **x, unsigned n_columns, unsigned width, unsigned perc
                         if (!e)
                                 return log_oom();
 
-                        printf("%4zu) %-*s", j * per_column + i + 1, width, e);
+                        printf("%4zu) %-*s", j * per_column + i + 1, (int) width, e);
                 }
 
                 putchar('\n');
@@ -259,8 +262,10 @@ static int prompt_locale(void) {
                 return 0;
         }
 
-        if (!arg_prompt_locale)
+        if (!arg_prompt_locale) {
+                log_debug("Prompting for locale was not requested.");
                 return 0;
+        }
 
         r = get_locales(&locales);
         if (r < 0)
@@ -312,8 +317,11 @@ static int process_locale(void) {
         int r;
 
         etc_localeconf = prefix_roota(arg_root, "/etc/locale.conf");
-        if (laccess(etc_localeconf, F_OK) >= 0 && !arg_force)
+        if (laccess(etc_localeconf, F_OK) >= 0 && !arg_force) {
+                log_debug("Found %s, assuming locale information has been configured.",
+                          etc_localeconf);
                 return 0;
+        }
 
         if (arg_copy_locale && arg_root) {
 
@@ -366,12 +374,14 @@ static int prompt_keymap(void) {
                 return 0;
         }
 
-        if (!arg_prompt_keymap)
+        if (!arg_prompt_keymap) {
+                log_debug("Prompting for keymap was not requested.");
                 return 0;
+        }
 
         r = get_keymaps(&kmaps);
         if (r == -ENOENT) /* no keymaps installed */
-                return r;
+                return log_debug_errno(r, "No keymaps are installed.");
         if (r < 0)
                 return log_error_errno(r, "Failed to read keymaps: %m");
 
@@ -387,8 +397,11 @@ static int process_keymap(void) {
         int r;
 
         etc_vconsoleconf = prefix_roota(arg_root, "/etc/vconsole.conf");
-        if (laccess(etc_vconsoleconf, F_OK) >= 0 && !arg_force)
+        if (laccess(etc_vconsoleconf, F_OK) >= 0 && !arg_force) {
+                log_debug("Found %s, assuming console has been configured.",
+                          etc_vconsoleconf);
                 return 0;
+        }
 
         if (arg_copy_keymap && arg_root) {
 
@@ -445,8 +458,10 @@ static int prompt_timezone(void) {
                 return 0;
         }
 
-        if (!arg_prompt_timezone)
+        if (!arg_prompt_timezone) {
+                log_debug("Prompting for timezone was not requested.");
                 return 0;
+        }
 
         r = get_timezones(&zones);
         if (r < 0)
@@ -467,8 +482,11 @@ static int process_timezone(void) {
         int r;
 
         etc_localtime = prefix_roota(arg_root, "/etc/localtime");
-        if (laccess(etc_localtime, F_OK) >= 0 && !arg_force)
+        if (laccess(etc_localtime, F_OK) >= 0 && !arg_force) {
+                log_debug("Found %s, assuming timezone has been configured.",
+                          etc_localtime);
                 return 0;
+        }
 
         if (arg_copy_timezone && arg_root) {
                 _cleanup_free_ char *p = NULL;
@@ -512,8 +530,10 @@ static int prompt_hostname(void) {
         if (arg_hostname)
                 return 0;
 
-        if (!arg_prompt_hostname)
+        if (!arg_prompt_hostname) {
+                log_debug("Prompting for hostname was not requested.");
                 return 0;
+        }
 
         print_welcome();
         putchar('\n');
@@ -549,8 +569,11 @@ static int process_hostname(void) {
         int r;
 
         etc_hostname = prefix_roota(arg_root, "/etc/hostname");
-        if (laccess(etc_hostname, F_OK) >= 0 && !arg_force)
+        if (laccess(etc_hostname, F_OK) >= 0 && !arg_force) {
+                log_debug("Found %s, assuming hostname has been configured.",
+                          etc_hostname);
                 return 0;
+        }
 
         r = prompt_hostname();
         if (r < 0)
@@ -574,11 +597,16 @@ static int process_machine_id(void) {
         int r;
 
         etc_machine_id = prefix_roota(arg_root, "/etc/machine-id");
-        if (laccess(etc_machine_id, F_OK) >= 0 && !arg_force)
+        if (laccess(etc_machine_id, F_OK) >= 0 && !arg_force) {
+                log_debug("Found %s, assuming machine-id has been configured.",
+                          etc_machine_id);
                 return 0;
+        }
 
-        if (sd_id128_is_null(arg_machine_id))
+        if (sd_id128_is_null(arg_machine_id)) {
+                log_debug("Initialization of machine-id was not requested, skipping.");
                 return 0;
+        }
 
         r = write_string_file(etc_machine_id, SD_ID128_TO_STRING(arg_machine_id),
                               WRITE_STRING_FILE_CREATE | WRITE_STRING_FILE_SYNC | WRITE_STRING_FILE_MKDIR_0755 |
@@ -597,25 +625,14 @@ static int prompt_root_password(void) {
         if (arg_root_password)
                 return 0;
 
-        r = read_credential("passwd.hashed-password.root", (void**) &arg_root_password, NULL);
-        if (r == -ENOENT) {
-                r = read_credential("passwd.plaintext-password.root", (void**) &arg_root_password, NULL);
-                if (r < 0)
-                        log_debug_errno(r, "Couldn't read credential 'passwd.{hashed|plaintext}-password.root', ignoring: %m");
-                else {
-                        arg_root_password_is_hashed = false;
-                        return 0;
-                }
-        } else if (r < 0)
-                log_debug_errno(r, "Couldn't read credential 'passwd.hashed-password.root', ignoring: %m");
-        else {
-                arg_root_password_is_hashed = true;
+        if (get_credential_user_password("root", &arg_root_password, &arg_root_password_is_hashed) >= 0)
+                return 0;
+
+        if (!arg_prompt_root_password) {
+                log_debug("Prompting for root password was not requested.");
                 return 0;
         }
 
-        if (!arg_prompt_root_password)
-                return 0;
-
         print_welcome();
         putchar('\n');
 
@@ -641,8 +658,12 @@ static int prompt_root_password(void) {
                 }
 
                 r = quality_check_password(*a, "root", &error);
-                if (r < 0)
-                        return log_error_errno(r, "Failed to check quality of password: %m");
+                if (r < 0) {
+                        if (ERRNO_IS_NOT_SUPPORTED(r))
+                                log_warning("Password quality check is not supported, proceeding anyway.");
+                        else
+                                return log_error_errno(r, "Failed to check password quality: %m");
+                }
                 if (r == 0)
                         log_warning("Password is weak, accepting anyway: %s", error);
 
@@ -697,8 +718,10 @@ static int prompt_root_shell(void) {
                 return 0;
         }
 
-        if (!arg_prompt_root_shell)
+        if (!arg_prompt_root_shell) {
+                log_debug("Prompting for root shell was not requested.");
                 return 0;
+        }
 
         print_welcome();
         putchar('\n');
@@ -768,7 +791,7 @@ static int write_root_passwd(const char *passwd_path, const char *password, cons
                         .pw_gid = 0,
                         .pw_gecos = (char *) "Super User",
                         .pw_dir = (char *) "/root",
-                        .pw_shell = (char *) (shell ?: "/bin/sh"),
+                        .pw_shell = (char *) (shell ?: default_root_shell(arg_root)),
                 };
 
                 if (errno != ENOENT)
@@ -863,8 +886,8 @@ static int write_root_shadow(const char *shadow_path, const char *hashed_passwor
         return 0;
 }
 
-static int process_root_args(void) {
-        _cleanup_close_ int lock = -1;
+static int process_root_account(void) {
+        _cleanup_close_ int lock = -EBADF;
         _cleanup_(erase_and_freep) char *_hashed_password = NULL;
         const char *password, *hashed_password;
         const char *etc_passwd, *etc_shadow;
@@ -873,17 +896,18 @@ static int process_root_args(void) {
         etc_passwd = prefix_roota(arg_root, "/etc/passwd");
         etc_shadow = prefix_roota(arg_root, "/etc/shadow");
 
-        /* We only mess with passwd and shadow if both do not exist or --force is specified. These files are
-         * tightly coupled and hence we make sure we have permission from the user to create/modify both
-         * files. */
-        if ((laccess(etc_passwd, F_OK) >= 0 || laccess(etc_shadow, F_OK) >= 0) && !arg_force)
+        if (laccess(etc_passwd, F_OK) >= 0 && laccess(etc_shadow, F_OK) >= 0 && !arg_force) {
+                log_debug("Found %s and %s, assuming root account has been initialized.",
+                          etc_passwd, etc_shadow);
                 return 0;
+        }
+
         /* Don't create/modify passwd and shadow if not asked */
         if (!(arg_root_password || arg_prompt_root_password || arg_copy_root_password || arg_delete_root_password ||
-              arg_root_shell || arg_prompt_root_shell || arg_copy_root_shell))
+              arg_root_shell || arg_prompt_root_shell || arg_copy_root_shell)) {
+                log_debug("Initialization of root account was not requested, skipping.");
                 return 0;
-
-        (void) mkdir_parents(etc_passwd, 0755);
+        }
 
         lock = take_etc_passwd_lock(arg_root);
         if (lock < 0)
@@ -960,11 +984,16 @@ static int process_kernel_cmdline(void) {
         int r;
 
         etc_kernel_cmdline = prefix_roota(arg_root, "/etc/kernel/cmdline");
-        if (laccess(etc_kernel_cmdline, F_OK) >= 0 && !arg_force)
+        if (laccess(etc_kernel_cmdline, F_OK) >= 0 && !arg_force) {
+                log_debug("Found %s, assuming kernel has been configured.",
+                          etc_kernel_cmdline);
                 return 0;
+        }
 
-        if (!arg_kernel_cmdline)
+        if (!arg_kernel_cmdline) {
+                log_debug("Creation of /etc/kernel/cmdline was not requested, skipping.");
                 return 0;
+        }
 
         r = write_string_file(etc_kernel_cmdline, arg_kernel_cmdline,
                               WRITE_STRING_FILE_CREATE | WRITE_STRING_FILE_SYNC | WRITE_STRING_FILE_MKDIR_0755 |
@@ -1326,7 +1355,6 @@ static int parse_argv(int argc, char *argv[]) {
 
 static int run(int argc, char *argv[]) {
         _cleanup_(loop_device_unrefp) LoopDevice *loop_device = NULL;
-        _cleanup_(decrypted_image_unrefp) DecryptedImage *decrypted_image = NULL;
         _cleanup_(umount_and_rmdir_and_freep) char *unlink_dir = NULL;
         int r;
 
@@ -1348,8 +1376,10 @@ static int run(int argc, char *argv[]) {
                 r = proc_cmdline_get_bool("systemd.firstboot", &enabled);
                 if (r < 0)
                         return log_error_errno(r, "Failed to parse systemd.firstboot= kernel command line argument, ignoring: %m");
-                if (r > 0 && !enabled)
+                if (r > 0 && !enabled) {
+                        log_debug("Found systemd.firstboot=no kernel command line argument, terminating.");
                         return 0; /* disabled */
+                }
         }
 
         if (arg_image) {
@@ -1364,8 +1394,7 @@ static int run(int argc, char *argv[]) {
                                 DISSECT_IMAGE_FSCK |
                                 DISSECT_IMAGE_GROWFS,
                                 &unlink_dir,
-                                &loop_device,
-                                &decrypted_image);
+                                &loop_device);
                 if (r < 0)
                         return r;
 
@@ -1394,7 +1423,7 @@ static int run(int argc, char *argv[]) {
         if (r < 0)
                 return r;
 
-        r = process_root_args();
+        r = process_root_account();
         if (r < 0)
                 return r;
 
